@@ -30,8 +30,6 @@
 
 #include "gdbus.h"
 
-#define DISPATCH_TIMEOUT  0
-
 #define info(fmt...)
 #define error(fmt...)
 #define debug(fmt...)
@@ -70,8 +68,6 @@ static gboolean message_dispatch(void *data)
 {
 	DBusConnection *conn = data;
 
-	dbus_connection_ref(conn);
-
 	/* Dispatch messages */
 	while (dbus_connection_dispatch(conn) == DBUS_DISPATCH_DATA_REMAINS);
 
@@ -84,7 +80,7 @@ static inline void queue_dispatch(DBusConnection *conn,
 						DBusDispatchStatus status)
 {
 	if (status == DBUS_DISPATCH_DATA_REMAINS)
-		g_timeout_add(DISPATCH_TIMEOUT, message_dispatch, conn);
+		g_idle_add(message_dispatch, dbus_connection_ref(conn));
 }
 
 static gboolean watch_func(GIOChannel *chan, GIOCondition cond, gpointer data)
@@ -93,8 +89,6 @@ static gboolean watch_func(GIOChannel *chan, GIOCondition cond, gpointer data)
 	unsigned int flags = 0;
 	DBusDispatchStatus status;
 
-	dbus_connection_ref(info->conn);
-
 	if (cond & G_IO_IN)  flags |= DBUS_WATCH_READABLE;
 	if (cond & G_IO_OUT) flags |= DBUS_WATCH_WRITABLE;
 	if (cond & G_IO_HUP) flags |= DBUS_WATCH_HANGUP;
@@ -105,8 +99,6 @@ static gboolean watch_func(GIOChannel *chan, GIOCondition cond, gpointer data)
 	status = dbus_connection_get_dispatch_status(info->conn);
 	queue_dispatch(info->conn, status);
 
-	dbus_connection_unref(info->conn);
-
 	return TRUE;
 }
 
