@@ -1,7 +1,7 @@
 /*
  *  DHCP library with GLib integration
  *
- *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2013  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -172,10 +172,7 @@ uint8_t *dhcpv6_get_option(struct dhcpv6_packet *packet, uint16_t pkt_len,
 		if (opt_code == code) {
 			if (option_len)
 				*option_len = opt_len;
-			if (rem < 0)
-				goto bad_packet;
-			else
-				found = optionptr + 2 + 2;
+			found = optionptr + 2 + 2;
 			count++;
 		}
 
@@ -371,34 +368,6 @@ void dhcpv6_init_header(struct dhcpv6_packet *packet, uint8_t type)
 	packet->transaction_id[2] = id & 0xff;
 }
 
-static bool check_vendor(uint8_t  *option_vendor, const char *vendor)
-{
-	uint8_t vendor_length = sizeof(vendor) - 1;
-
-	if (option_vendor[OPT_LEN - OPT_DATA] != vendor_length)
-		return false;
-
-	if (memcmp(option_vendor, vendor, vendor_length) != 0)
-		return false;
-
-	return true;
-}
-
-static void check_broken_vendor(struct dhcp_packet *packet)
-{
-	uint8_t *vendor;
-
-	if (packet->op != BOOTREQUEST)
-		return;
-
-	vendor = dhcp_get_option(packet, DHCP_VENDOR);
-	if (!vendor)
-		return;
-
-	if (check_vendor(vendor, "MSFT 98"))
-		packet->flags |= htons(BROADCAST_FLAG);
-}
-
 int dhcp_recv_l3_packet(struct dhcp_packet *packet, int fd)
 {
 	int n;
@@ -412,8 +381,6 @@ int dhcp_recv_l3_packet(struct dhcp_packet *packet, int fd)
 	if (packet->cookie != htonl(DHCP_MAGIC))
 		return -EPROTO;
 
-	check_broken_vendor(packet);
-
 	return n;
 }
 
@@ -562,6 +529,8 @@ int dhcp_send_raw_packet(struct dhcp_packet *dhcp_pkt,
 	if (fd < 0)
 		return -errno;
 
+	dhcp_pkt->flags |= htons(BROADCAST_FLAG);
+
 	memset(&dest, 0, sizeof(dest));
 	memset(&packet, 0, sizeof(packet));
 	packet.data = *dhcp_pkt;
